package vehicle;

import java.util.ArrayList;

import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;
import org.orekit.errors.OrekitException;
import org.orekit.frames.Frame;
import org.orekit.frames.FramesFactory;

import fr.cnes.genopus.nonWidget.exception.NonWidgetException;
import fr.cnes.genopus.nonWidget.maneuvers.CustomEngine;
import fr.cnes.genopus.nonWidget.maneuvers.CustomFuelTank;
import fr.cnes.genopus.nonWidget.vehicle.CustomAerodynamicProperties;
import fr.cnes.genopus.nonWidget.vehicle.CustomRadiativeProperties;
import fr.cnes.genopus.nonWidget.vehicle.CustomVehicle;
import fr.cnes.genopus.nonWidget.vehicle.CustomVehicleSurfaceModel;
import fr.cnes.genopus.nonWidget.vehicle.geometry.CustomCylinder;
import fr.cnes.genopus.nonWidget.vehicle.geometry.CustomParallelepiped;
import fr.cnes.genopus.nonWidget.vehicle.geometry.CustomSphere;
import fr.cnes.sirius.patrius.assembly.Assembly;
import fr.cnes.sirius.patrius.assembly.properties.MassProperty;
import fr.cnes.sirius.patrius.assembly.properties.RadiativeIRProperty;
import fr.cnes.sirius.patrius.assembly.properties.RadiativeProperty;

/**
 * Tutorial for using CustomVehicle.
 * @author goesterjf
 *
 */
public final class TestCustomVehicle {
    
    /** Definition of a shape corresponding to a sphere without solar panels */
    private static final int SPHERE_NOSP = 0;
    /** Definition of a shape corresponding to a sphere with solar panels */
    private static final int SPHERE = 1;
    /** Definition of a shape corresponding to a parallelepiped with solar panels */
    private static final int PARALLELEPIPED = 2;
    /** Definition of a shape corresponding to a cylinder with solar panels */
    private static final int CYLINDER = 3;
    
    /**
     * Constructor.
     */
    private TestCustomVehicle () {
        
    }
    
    /**
     * Main executable method
     * @param args no args ...
     */
    public static void main(String[] args) {

        // DRY MASS
        final double dryMass = 1000.;
        MassProperty dryMassProperty = null;
        try {
            dryMassProperty = new MassProperty(dryMass);
        } catch (OrekitException err) {
            err.printStackTrace();
        }
        
        // PROPULSION
        final ArrayList<CustomEngine> enginesList = new ArrayList<CustomEngine>();
        enginesList.add(new CustomEngine("OCS", 320., 1000.));
        enginesList.add(new CustomEngine("ACS", 150., 270.));
        final ArrayList<CustomFuelTank> fuelTankList = new ArrayList<CustomFuelTank>();
        fuelTankList.add(new CustomFuelTank("Tank1", 500.));

        // SHAPE
        
        final int shape = PARALLELEPIPED;
        CustomVehicleSurfaceModel vehicleRefSurface = null;
        final CustomParallelepiped solarPanels = new CustomParallelepiped(0., 0., 0.); // To be replaced by addSolarPanels ?
        
        switch (shape) {
            case SPHERE_NOSP:
                // Sphere case without solar panels
                final CustomSphere sphere = new CustomSphere(5.0);
                vehicleRefSurface = new CustomVehicleSurfaceModel(sphere);
                break;
            case SPHERE:
                // Sphere case + solar panels
                final CustomSphere sphere2 = new CustomSphere(5.0);
                vehicleRefSurface = new CustomVehicleSurfaceModel(sphere2, solarPanels);
                break;
            case PARALLELEPIPED:
                // Parallelepiped case + solar panels
                final CustomParallelepiped parall = new CustomParallelepiped(4.0, 1.0, 2.0);
                vehicleRefSurface = new CustomVehicleSurfaceModel(parall, solarPanels);
                break;
            case CYLINDER:
                // Cylinder case + solar panels
                final CustomCylinder cylinder = new CustomCylinder(4.0, 10.0);
                vehicleRefSurface = new CustomVehicleSurfaceModel(cylinder, solarPanels);
                break;
            default:
                System.out.println("No defined shape ...");
                break;
        }
        
        // AERODYNAMIC PROPERTIES
        
        final double cd = 2.0;
        final double cl = 0.;
        final CustomAerodynamicProperties aerodynamicProperties = new CustomAerodynamicProperties(vehicleRefSurface, cd, cl);
        
        // RADIATIVE PROPERTIES
        
        final double ka = 1.0;
        final double ks = 0.0;
        final double kd = 0.0;
        final RadiativeProperty radiativeProperty = new RadiativeProperty(ka, ks, kd);
        final double absorptionCoef = 1.0;
        final double specularCoef = 0.0;
        final double diffuseCoef = 0.0;
        final RadiativeIRProperty radiativeIRProperty =
                new RadiativeIRProperty(absorptionCoef, specularCoef, diffuseCoef);
        final CustomRadiativeProperties radiativeProperties =
                new CustomRadiativeProperties(radiativeProperty, radiativeIRProperty, vehicleRefSurface);
        
        final CustomVehicle vehicle = new CustomVehicle(dryMassProperty, aerodynamicProperties, radiativeProperties, enginesList, fuelTankList);
        
        final double massMultiplicativeFactor = 1.0;
        final double aeroSurfMultiplicativeFactor = 1.0;
        final double radSurfMultiplicativeFactor = 1.0;
        final Frame gcrf = FramesFactory.getGCRF();
//        try {
            final Assembly assembly =
                    vehicle.getAssembly(massMultiplicativeFactor, aeroSurfMultiplicativeFactor, radSurfMultiplicativeFactor, gcrf);
            System.out.println("List of Part names ...");
            for (String name : assembly.getAllPartsNames()) {
                System.out.println(name);
            }
//        } catch (NonWidgetException err) {
//            err.printStackTrace();
//        }
        
        System.out.println("\nMASS PROPERTIES");
        System.out.println("DRY MASS = "+vehicle.getDryMass()+" kg");
        System.out.println("ERG MASS = "+vehicle.getErgolsMass()+" kg");
        System.out.println("TOT MASS = "+vehicle.getTotalMass()+" kg");
        
        System.out.println("\nPROPULSIVE PROPERTIES");
        System.out.println("NB ENGINES = "+vehicle.getEnginesList().size());
        System.out.println("NB TANKS = "+vehicle.getTanksList().size());
        
        System.out.println("\nAERODYNAMIC PROPERTIES");
        final Vector3D xDir = new Vector3D(1., 0., 0.);
        final Vector3D yDir = new Vector3D(0., 1., 0.);
        final Vector3D zDir = new Vector3D(0., 0., 1.);
        System.out.println("SX = "+vehicle.getAerodynamicProperties().getVehicleSurfaceModel().getMainPartShape().getCrossSection(xDir)+" m2");
        System.out.println("SY = "+vehicle.getAerodynamicProperties().getVehicleSurfaceModel().getMainPartShape().getCrossSection(yDir)+" m2");
        System.out.println("SZ = "+vehicle.getAerodynamicProperties().getVehicleSurfaceModel().getMainPartShape().getCrossSection(zDir)+" m2");
        try {
            System.out.println("CD = "+vehicle.getAerodynamicProperties().getConstantDragCoef());
        } catch (NonWidgetException err) {
            err.printStackTrace();
        }

        System.out.println("\nRADIATIVE PROPERTIES");
        System.out.println("SX = "+vehicle.getRadiativeProperties().getVehicleSurfaceModel().getMainPartShape().getCrossSection(xDir)+" m2");
        System.out.println("SY = "+vehicle.getRadiativeProperties().getVehicleSurfaceModel().getMainPartShape().getCrossSection(yDir)+" m2");
        System.out.println("SZ = "+vehicle.getRadiativeProperties().getVehicleSurfaceModel().getMainPartShape().getCrossSection(zDir)+" m2");
        System.out.println("KA = "+vehicle.getRadiativeProperties().getRadiativeProperty().getAbsorptionRatio().getValue());
        System.out.println("KD = "+vehicle.getRadiativeProperties().getRadiativeProperty().getDiffuseReflectionRatio().getValue());
        System.out.println("KS = "+vehicle.getRadiativeProperties().getRadiativeProperty().getSpecularReflectionRatio().getValue());
        System.out.println("KAI = "+vehicle.getRadiativeProperties().getRadiativeIRProperty().getAbsorptionCoef().getValue());
        System.out.println("KDI = "+vehicle.getRadiativeProperties().getRadiativeIRProperty().getDiffuseReflectionCoef().getValue());
        System.out.println("KSI = "+vehicle.getRadiativeProperties().getRadiativeIRProperty().getSpecularReflectionCoef().getValue());

        
    }

}
